GroundProximityDetection = true

config = {
	["av_8"] = {VTOL = true, weaponLoc = Vec(-0.55, 0.65, -2.9)},
	["f_35b"] = {VTOL = true, weaponLoc = Vec(0.05, 0.6, -4.6)},
	["su_47"] = {weaponLoc = Vec(-1.15, 3.3, -4.1)}
}

function EulerQuat(q, k)
	local x, y, z, w = q[1], q[2], q[3], q[4]
	local bank, heading, attitude

	local s = 2 * x * y + 2 * z * w
	if s >= 1 then
		bank = 0
		heading = 2 * math.atan2(x, w)
		attitude = math.pi / 2
	elseif s <= -1 then
		bank = 0
		heading = -2 * math.atan2(x, w)
		attitude = math.pi / -2
	else
		bank = math.atan2(2 * x * w - 2 * y * z, 1 - 2 * x ^ 2 - 2 * z ^ 2)
		heading = math.atan2(2 * y * w - 2 * x * z, 1 - 2 * y ^ 2 - 2 * z ^ 2)
		attitude = math.asin(s)
	end
	bank, heading, attitude = math.deg(bank), math.deg(heading), math.deg(attitude)

	if k and k > 90 and k < 270 then
		bank, heading, attitude = bank - 180, heading - 180, 180 - attitude
	elseif k and k >= 270 then
		attitude = attitude + 360
	elseif k and k < -90 then
		bank, heading, attitude = bank - 180, heading - 180, -(attitude + 180)
	end
	return bank, heading, attitude
end

function clamp(value, mi, ma)
	if value < mi then value = mi end
	if value > ma then value = ma end
	return value
end

function sign(x)
	if x < 0 then
		return -1
	elseif x > 0 then
		return 1
	else
		return 0
	end
end

function YawRight(yaw)
	yaw = yaw - 0.5
	if yaw < -180 then
		yaw = yaw + 360
	end
	return yaw
end

function YawLeft(yaw)
	yaw = yaw + 0.5
	if yaw > 180 then
		yaw = yaw - 360
	end
	return yaw
end

function init()
	aircraft = {}
	aircraft.body = FindBody("aircraft_body")
	aircraft.transform = GetBodyTransform(aircraft.body)
	aircraft.staticBlock = FindBody("static_cmd")
	aircraft.staticBlockLocalTransform = TransformToLocalTransform(aircraft.transform, GetBodyTransform(aircraft.staticBlock))

	aircraft.angle = 0.0
	aircraft.mass = GetBodyMass(aircraft.body)
	aircraft.sound = LoadLoop("aircraft-loop.ogg")
	aircraft.vtolSound = LoadLoop("hover-loop.ogg")

	shootSound = LoadSound("chopper-shoot0.ogg")
	--rocketSound = LoadSound("tools/launcher0.ogg")

	pitch = 0
	yaw = 0
	roll = 0
	collective = aircraft.transform.pos[2] + 0.5

	vehicleType = GetTagValue(aircraft.body, "aircraft_body")
	if config[vehicleType].VTOL then
		vtolEnabled = true
	end
	if config[vehicleType].weaponLoc then
		weaponLoc = config[vehicleType].weaponLoc
	end

	cameraX = 0
	cameraY = 0
	zoom = 20

	hover = false
	dynamicRotor = true
	aimMode = false
	deltaY = 0
	throttle = 0

	dynamicRotor = true
	modeUpdated = false
	vehicleMode = "Drive"
end


function tick(dt)
	local vehicle = GetPlayerVehicle()
	if vehicle ~= FindVehicle("aircraft") then
		if not dynamicRotor then -- Player leaves the aircraft
			dynamicRotor = true
			SetBodyDynamic(aircraft.staticBlock, true)
			SetBodyAngularVelocity(aircraft.body, Vec())
			SetPlayerVehicle(0)
			pitch = 0
			roll = 0
			cameraY = pitch
			local playerSpawn = TransformToParentPoint(aircraft.transform, Vec(2, 0, -4))
			local playerRot = QuatEuler(pitch, yaw, roll)
			--SetPlayerTransform(Transform(playerSpawn, playerRot))
		end
		return
	elseif GetBodyMass(aircraft.body) < aircraft.mass * 0.66 then
		if not dynamicRotor then -- Aircraft is too damaged
			dynamicRotor = true
			SetBodyDynamic(aircraft.staticBlock, true)
			vehicleMode = "Drive"
		end
		return
	else
		if InputPressed("c") then
			if vtolEnabled then
				if vehicleMode == "Aircraft" then
					vehicleMode = "VTOL"
				elseif vehicleMode == "VTOL" then
					vehicleMode = "Drive"
				else
					vehicleMode = "Aircraft"
				end
			else
				if vehicleMode == "Aircraft" then
					vehicleMode = "Drive"
				else
					vehicleMode = "Aircraft"
				end
			end
			modeUpdated = true
		end
		if InputPressed("v") then
			if vtolEnabled then
				if vehicleMode == "Drive" then
					vehicleMode = "VTOL"
				elseif vehicleMode == "VTOL" then
					vehicleMode = "Aircraft"
				else
					vehicleMode = "Drive"
				end
			else
				if vehicleMode == "Aircraft" then
					vehicleMode = "Drive"
				else
					vehicleMode = "Aircraft"
				end
			end
			modeUpdated = true
		end
		if vehicleMode ~= "Drive" then
			if dynamicRotor then
				dynamicRotor = false
				SetBodyDynamic(aircraft.staticBlock, false)
			end
			if modeUpdated then
				modeUpdated = false
				aircraft.transform.pos = GetBodyTransform(aircraft.body).pos
				local unused = 0
				unused, yaw = EulerQuat(GetBodyTransform(aircraft.body).rot)
				cameraX = yaw
				aircraft.transform.rot = QuatEuler(5, yaw, 0)
				collective = aircraft.transform.pos[2] + 0.5
				local vel = GetBodyVelocity(aircraft.body)
				vel = TransformToParentVec(aircraft.transform, vel)
				throttle = clamp(vel[3] / 20, 0.3, 1.2)
			end
		else
			if not dynamicRotor then
				dynamicRotor = true
				SetBodyDynamic(aircraft.staticBlock, true)
			end
			return
		end
	end

	if weaponLoc then
		local firePos = TransformToParentPoint(aircraft.transform, weaponLoc)
		local fireDir = TransformToParentVec(aircraft.transform, Vec(0, 0, -1))
		if InputPressed("lmb") then
			PlaySound(shootSound, aircraft.transform.pos, 5)
			Shoot(firePos, fireDir, 0)
		end
	end

	if vehicleMode == "VTOL" then

	if InputDown("w") then
		collective = collective + 0.1
	end
	if InputDown("s") and collective > 0 then
		collective = collective - 0.1
	end

	if InputDown("a") and roll < 30 then
		roll = roll + 0.5
	end
	if InputDown("d") and roll > -30 then
		roll = roll - 0.5
	end

	if InputPressed("h") then
		hover = not hover
	end

	local prevYaw = yaw

	if not aimMode then
		local x, y = EulerQuat(GetCameraTransform().rot)

		if x < pitch and pitch > -45 then
			pitch = pitch - 0.5
		end
		if x > pitch and pitch < 15 then
			pitch = pitch + 0.5
		end

		if y < yaw and not (y < yaw - 180) then
			yaw = YawRight(yaw)
		elseif y < yaw - 180 then
			yaw = YawLeft(yaw)
		end

		if y > yaw and not (y > yaw + 180) then
			yaw = YawLeft(yaw)
		elseif y > yaw + 180 then
			yaw = YawRight(yaw)
		end
	else
		local mouseX = -InputValue("mousedx")
		local mouseY = -InputValue("mousedy")

		if sign(mouseY) < 0 and pitch > -45 then
			pitch = pitch - 0.5
		elseif sign(mouseY) > 0 and pitch < 15 then
			pitch = pitch + 0.5
		end

		if sign(mouseX) < 0 then
			yaw = YawRight(yaw)
		elseif sign(mouseX) > 0 then
			yaw = YawLeft(yaw)
		end
	end

	if hover then
		if pitch ~= 0 then
			pitch = pitch - sign(pitch)
		end
		yaw = prevYaw
	end

	local moveDir = Vec(0.01 * -roll, 0, 0.01 * pitch)
	local pos = TransformToParentPoint(aircraft.transform, moveDir)

	if GroundProximityDetection then
		pos[2] = pos[2] - 0.1
		local dir = VecNormalize(VecSub(pos, aircraft.transform.pos))
		local maxDist = 0.5 + math.abs(pitch / 5) + math.abs(roll / 10)
		QueryRejectBody(aircraft.body)
		QueryRequire("physical large")
		local hit, dist = QueryRaycast(aircraft.transform.pos, dir, maxDist)
		--local posEnd = VecAdd(aircraft.transform.pos, VecScale(dir, maxDist))
		if hit then
			collective = collective + (maxDist - dist) / (2 * maxDist)
		end
	end
	pos[2] = collective

	aircraft.angle = aircraft.angle + 0.6
	local rot = QuatEuler(math.sin(aircraft.angle*0.053)*5 + pitch, math.sin(aircraft.angle*0.04)*5 + yaw, roll)

	if aimMode then
		rot = QuatEuler(pitch, yaw, roll)
	end

	aircraft.transform.pos = pos
	aircraft.transform.rot = rot
	SetBodyTransform(aircraft.body, aircraft.transform)
	SetBodyTransform(aircraft.staticBlock, TransformToParentTransform(aircraft.transform, aircraft.staticBlockLocalTransform))

	PlayLoop(aircraft.vtolSound, aircraft.transform.pos, 10)
	if vehicleType == "f_35b" then
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(0.05, 2.05, 5.8)), Vec(0, -5, 0), 1, 0.25)
	elseif vehicleType == "av_8" then
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(0.9, 1.2, 0.5)), Vec(0, -5, 0), 0.5, 0.25)
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(-0.8, 1.2, 0.5)), Vec(0, -5, 0), 0.5, 0.25)
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(1, 1.5, -1.2)), Vec(0, -5, 0), 0.5, 0.25)
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(-0.9, 1.5, -1.2)), Vec(0, -5, 0), 0.5, 0.25)
	end

	else
	
	if InputDown("w") then
		throttle = throttle + 0.005
	end
	if InputDown("s") then
		throttle = throttle - 0.005
	end
	throttle = clamp(throttle, 0.3, 1.2)

	if InputDown("a") then
		deltaY = deltaY + 0.1
	end
	if InputReleased("a") then
		deltaY = 0
	end
	if InputDown("d") then
		deltaY = deltaY - 0.1
	end
	if InputReleased("d") then
		deltaY = 0
	end
	deltaY = clamp(deltaY, -1, 1)

	local x, y = EulerQuat(GetCameraTransform().rot)
	local i, j = EulerQuat(aircraft.transform.rot)

	local rollDir = y - j
	if (y < -90 or y > 90) and ((y < 0 and j > 0) or (y > 0 and j < 0)) then
		-- FUCK THIS SHIT!!! ((((
		rollDir = -rollDir
	end

	local rotation = clamp(rollDir * 2, -85, 85)
	local moveDir = Transform(Vec(0, 0, -throttle), QuatEuler(0, deltaY, 0))
	aircraft.transform = TransformToParentTransform(aircraft.transform, moveDir)

	if GroundProximityDetection then
		QueryRejectBody(aircraft.body)
		QueryRequire("physical large")
		local maxDist = throttle * 20
		local front = TransformToParentVec(aircraft.transform, Vec(0, 0, -1))
		local hit, dist = QueryRaycast(aircraft.transform.pos, front, maxDist)
		--local posEnd = VecAdd(aircraft.transform.pos, VecScale(front, maxDist))
		if hit then
			x = maxDist - dist
		end
	end

	aircraft.transform.rot = QuatSlerp(aircraft.transform.rot, QuatEuler(x, y, rotation), 0.025)
	SetBodyTransform(aircraft.body, aircraft.transform)
	SetBodyTransform(aircraft.staticBlock, TransformToParentTransform(aircraft.transform, aircraft.staticBlockLocalTransform))

	PlayLoop(aircraft.sound, aircraft.transform.pos, 10)
	if vehicleType == "su_47" then
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(0.5, 2.1, 9)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 2, 0.5)
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(-0.5, 2.1, 9)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 2, 0.5)
	elseif vehicleType == "f_35b" then
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(0.05, 2.05, 5.8)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 2, 0.5)
	elseif vehicleType == "av_8" then
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(0.9, 1.2, 0.5)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 0.75, 0.2)
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(-0.8, 1.2, 0.5)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 0.75, 0.2)
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(1, 1.5, -1.2)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 0.75, 0.2)
		SpawnParticle("fire", TransformToParentPoint(aircraft.transform, Vec(-0.9, 1.5, -1.2)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 0.75, 0.2)
	end
	end

	local mx, my = InputValue("mousedx"), InputValue("mousedy")
	cameraX = cameraX - mx / 10
	cameraY = cameraY - my / 10
	cameraY = clamp(cameraY, -90, 90)
	local cameraRot = QuatEuler(cameraY, cameraX, 0)
	local cameraT = Transform(aircraft.transform.pos, cameraRot)
	zoom = zoom - InputValue("mousewheel")
	zoom = clamp(zoom, 10, 30)
	local cameraPos = TransformToParentPoint(cameraT, Vec(0, 5, zoom))
	local camera = Transform(cameraPos, cameraRot)
	SetCameraTransform(camera)
end

function draw()
	local vehicle = GetPlayerVehicle()
	if vehicle == FindVehicle("aircraft") then
		UiPush()
		UiFont("regular.ttf", 32)
		local h = UiHeight()
		UiTranslate(10, h - 10)
		UiText(vehicleMode .. " (C/V to change)")
		UiPop()
	end
end
