
function EulerQuat(q, k)
	local x, y, z, w = q[1], q[2], q[3], q[4]
	local bank, heading, attitude

	local s = 2 * x * y + 2 * z * w
	if s >= 1 then
		bank = 0
		heading = 2 * math.atan2(x, w)
		attitude = math.pi / 2
	elseif s <= -1 then
		bank = 0
		heading = -2 * math.atan2(x, w)
		attitude = math.pi / -2
	else
		bank = math.atan2(2 * x * w - 2 * y * z, 1 - 2 * x ^ 2 - 2 * z ^ 2)
		heading = math.atan2(2 * y * w - 2 * x * z, 1 - 2 * y ^ 2 - 2 * z ^ 2)
		attitude = math.asin(s)
	end
	bank, heading, attitude = math.deg(bank), math.deg(heading), math.deg(attitude)

	if k and k > 90 and k < 270 then
		bank, heading, attitude = bank - 180, heading - 180, 180 - attitude
	elseif k and k >= 270 then
		attitude = attitude + 360
	elseif k and k < -90 then
		bank, heading, attitude = bank - 180, heading - 180, -(attitude + 180)
	end
	return bank, heading, attitude
end

function clamp(value, mi, ma)
	if value < mi then value = mi end
	if value > ma then value = ma end
	return value
end

function sign(x)
	if x < 0 then
		return -1
	elseif x > 0 then
		return 1
	else
		return 0
	end
end

function init()
	aircraft = {}
	aircraft.body = FindBody("aircraft_body")
	aircraft.mass = GetBodyMass(aircraft.body)

	shootSound = LoadSound("chopper-shoot0.ogg")
	aircraft.sound = LoadLoop("aircraft-loop.ogg")

	pitch = 0
	yaw = 0
	roll = 0
	throttle = 0

	cameraX = 0
	cameraY = 0
	zoom = 20
end

function tick(dt)
	local vehicle = GetPlayerVehicle()
	if vehicle ~= FindVehicle("aircraft") or GetBodyMass(aircraft.body) < aircraft.mass * 0.66 then
		throttle = 0
		return
	end

	local aircraftTransform = GetBodyTransform(aircraft.body)
	local bank, heading, attitude = EulerQuat(aircraftTransform.rot)
	local my, mx = EulerQuat(GetCameraTransform().rot)

	local firePos = TransformToParentPoint(aircraftTransform, Vec(-1.15, 3.3, -4.1))
	local fireDir = TransformToParentVec(aircraftTransform, Vec(0, 0, -1))
	if InputDown("lmb") then
		PlaySound(shootSound, aircraftTransform.pos, 2)
		Shoot(firePos, fireDir, 0)
	end

	if InputDown("r") then
		throttle = throttle + 0.7
	end
	if InputDown("f") then
		throttle = throttle - 0.5
	end
	throttle = clamp(throttle, 10, 70)

	my = my + 10
	if my < bank and bank > -85 then
		pitch = pitch - 0.1
	elseif my > bank and bank < 85 then
		pitch = pitch + 0.1
	elseif bank <= -85 or bank >= 85 then
		pitch = -0.1 * sign(bank)
	else
		pitch = 0
	end
	if math.abs(my - bank) < 5 then
		pitch = 0
	end
	pitch = clamp(pitch, -1, 1)

	if mx < heading and not (mx < heading - 180) then
		yaw = yaw - 0.1
	elseif mx < heading - 180 then
		yaw = yaw + 0.1
	elseif mx > heading and not (mx > heading + 180) then
		yaw = yaw + 0.1
	elseif mx > heading + 180 then
		yaw = yaw - 0.1
	end
	if math.abs(mx - heading) < 5 or math.abs(mx - heading) > 355 then
		yaw = 0
	end
	yaw = clamp(yaw, -0.5, 0.5)

	if yaw > 0 and attitude < 85 then
		roll = roll + 0.1
	elseif yaw < 0 and attitude > -85 then
		roll = roll - 0.1
	else
		roll = -0.5 * sign(attitude)
	end
	if math.abs(attitude) < 2 then
		roll = 0
	end
	roll = clamp(roll, -1, 1)

	local velocity = Vec(0, 0, -throttle)
	local moveDir = TransformToParentVec(aircraftTransform, velocity)
	local angVel = Vec(pitch, yaw, roll)
	local spinDir = TransformToParentVec(aircraftTransform, angVel)
	if throttle > 15 then
		SetBodyVelocity(aircraft.body, moveDir)
		SetBodyAngularVelocity(aircraft.body, spinDir)
		PlayLoop(aircraft.sound, aircraftTransform.pos, 10)
	end

	SpawnParticle("fire", TransformToParentPoint(aircraftTransform, Vec(0.5, 2.1, 9)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 2, 0.5)
	SpawnParticle("fire", TransformToParentPoint(aircraftTransform, Vec(-0.5, 2.1, 9)), TransformToParentVec(aircraftTransform, Vec(0, 0, 1)), 2, 0.5)

	mx, my = InputValue("mousedx"), InputValue("mousedy")
	cameraX = cameraX - mx / 10
	cameraY = cameraY - my / 10
	cameraY = clamp(cameraY, -90, 90)
	local cameraRot = QuatEuler(cameraY, cameraX, 0)
	local cameraT = Transform(aircraftTransform.pos, cameraRot)
	zoom = zoom - InputValue("mousewheel")
	zoom = clamp(zoom, 10, 30)
	local cameraPos = TransformToParentPoint(cameraT, Vec(0, 5, zoom))
	local camera = Transform(cameraPos, cameraRot)
	SetCameraTransform(camera)
end


function draw()
	local vehicle = GetPlayerVehicle()
	if vehicle == FindVehicle("aircraft") then
		UiPush()
		UiFont("regular.ttf", 32)
		local h = UiHeight()
		UiTranslate(10, h - 10)
		UiText("R/F for Throttle")
		UiPop()
	end
end
