#include "script/common.lua"
pAmmo = GetIntParam("ammo",3)
pGun = GetBoolParam("gun",false)

function init()
    cabmotor = FindJoint("cabmotor")
    armmotor = FindJoint("armmotor")    

	armSnd = LoadLoop("turretarm.ogg")
	hutSnd = LoadLoop("turrethut.ogg")

    armTurn = 0
    armSpeed = 0.3

    hutTurn = 0
    hutSpeed = 0.6

	armAngleMin, armAngleMax = GetJointLimits(armmotor)
	armAngle = GetJointMovement(armmotor)
    SetJointMotorTarget(armmotor, armAngle, 1.0, 1000.0)
    SetJointMotorTarget(cabmotor, 0, 1000.0)

	armCurrent = GetJointMovement(armmotor)
	hutCurrent = GetJointMovement(cabmotor)

    ammo = pAmmo
	reach = 25
	reloadTime = 0

	explosionSize = 1.0

	didImpact = false
	impactPoint = Vec(0,0,0)

	projectileTime = 0.0
	projectileTimeMax = 0.3
	projectilePos = Vec(0,0,0)
	projectileStartPos = Vec(0,0,0)
	projectileHitPos = Vec(0,0,0)

    direction = Vec(0,0,0)

    armBody = FindBody("arm")
    armTransform = GetBodyTransform(armBody)

    muzzleLocation = FindLocation("muzzle")
    muzzleTransform = GetLocationTransform(muzzleLocation)
    localMuzzlePos = TransformToLocalPoint(armTransform, muzzleTransform.pos)

    directionLocation = FindLocation("direction")
    directionTransform = GetLocationTransform(directionLocation)
    localDirectionPos = TransformToLocalPoint(armTransform, directionTransform.pos)

	shootSnd = LoadSound("tools/tankgun0.ogg")
	recoil = 0

	ai = true
end

fire = false

function tick(dt)
		ai = GetBool("level.ai_enabled")

        armTransform = GetBodyTransform(armBody)
        worldMuzzlePos = TransformToParentPoint(armTransform, localMuzzlePos)
        worldDirectionPos = TransformToParentPoint(armTransform, localDirectionPos)

        direction = VecSub(worldDirectionPos, worldMuzzlePos)
        direction = VecNormalize(direction)

        shoot(dt)
        if projectileTime > 0 then
            projectileTime = projectileTime - dt
            projectTilePos = VecLerp(projectileHitPos, projectileStartPos, projectileTime/projectileTimeMax)
			ParticleType("smoke")
			ParticleRadius(1)
            SpawnParticle(projectTilePos, VecScale(VecNormalize(VecSub(projectTilePos,projectileHitPos)),2), 0.5)
            PointLight(projectTilePos, 0.5, 0.5, 0.5, math.random(1,15))
        elseif didImpact then
            didImpact = false
            Explosion(impactPoint, explosionSize)
        end
end

function update(dt)
		local armTurnTarget = 0
		
		if not ai then
			if InputDown("k") then            
				armTurnTarget = -1.0
			elseif InputDown("i") then            
				armTurnTarget = 1.0
			end
		end

		delta = 0.75
		armTurn = armTurn + clamp(armTurnTarget-armTurn, -delta, delta)

		local current = GetJointMovement(armmotor)
		armAngle = clamp(armAngle, current-5, current+5)
		armAngle = clamp(armAngle + armTurn*armSpeed*180.0/math.pi*dt, armAngleMin, armAngleMax)

		local hutTurnTarget = 0

		if ai then
			if GetJointMovement(cabmotor) < -15 then
				move_left = true
			end
			if GetJointMovement(cabmotor) > 15 then
				move_left = false
			end
			if move_left then
				hutTurnTarget = -1.0
			else
				hutTurnTarget = 1.0
			end
			armAngle = -12
		else
			if InputDown("l") then            
				hutTurnTarget = 1.0
			elseif InputDown("j") then            
				hutTurnTarget = -1.0
			end
		end

		hutTurn = hutTurn + clamp(hutTurnTarget-hutTurn, -delta, delta)

		local ac = GetJointMovement(armmotor)
		local av = math.abs(armCurrent - ac)
		armCurrent = ac
		local hc = GetJointMovement(cabmotor)
		local hv = math.abs(hutCurrent - hc)
		hutCurrent = hc

		if av > 0.1 then PlayLoop(armSnd, armTransform.pos, av*3) end
		if hv > 0.1 then PlayLoop(hutSnd, armTransform.pos, hv*3) end

		SetJointMotorTarget(armmotor, armAngle, 1.0, 1000)
		SetJointMotor(cabmotor, hutTurn * hutSpeed, 1000)
end

function shoot(dt)
		if ai then
			reach = 25
			QueryRequire("dynamic large")
		else
			reach = 500
		end
		QueryRejectBody(armBody)
		local hit, dist = QueryRaycast(worldMuzzlePos, direction, reach)
		hitPoint = VecAdd(worldMuzzlePos, VecScale(direction, dist))
		projectileHitPos = VecCopy(hitPoint)
		
		fire = false
		if hit then
			DrawLine(worldMuzzlePos, hitPoint, 1, 0, 0, 0.2)
			PointLight(hitPoint,1, 0.1, 0.1, 0.2+0.1*math.random())
			if ai then
				fire = true
			end
		else
			DrawLine(worldMuzzlePos, VecAdd(worldMuzzlePos,VecScale(direction, reach)), 1, 0, 0, 0.2)
		end

		if not ai and InputPressed("lmb") then
			fire = true
		end

		if reloadTime > 0 then
			reloadTime = reloadTime - dt
			return
		end

		if fire then
			if ammo > 0 or GetInt("level.sandbox") == 1 then
				recoil = 1.0
				SetValue("recoil", 0, "easeout", 1.0)

				projectileTime = projectileTimeMax				
				projectilePos = VecCopy(worldMuzzlePos)
				projectileStartPos = VecCopy(worldMuzzlePos)

				PlaySound(shootSnd, worldMuzzlePos)
				reloadTime = 1
				ammo = ammo - 1

				ParticleType("smoke")
				ParticleRadius(2)
				SpawnParticle(worldMuzzlePos, VecScale(direction,3), 5)

				didImpact = hit

				if hit then
					hitPoint = VecAdd(worldMuzzlePos, VecScale(direction, dist))
					impactPoint = VecCopy(hitPoint)
					projectileHitPos = VecCopy(hitPoint)
				else
					projectileHitPos = VecAdd(worldMuzzlePos, VecScale(VecNormalize(direction), 50))
				end
			end
		end
end
