MAX_DIST = 100
DONE_TIMER = 2.0
BURN_TIME = 25
PLAYER_HIT_RADIUS = 0.4

function rnd(mi, ma)
	return math.random(1000)/1000*(ma-mi) + mi
end

function rndVec(t)
	return Vec(rnd(-t, t), rnd(-t, t), rnd(-t, t))
end 

function init()
	vehicle = FindVehicle()
	emitterShape = FindShape("emitter")

	laserLoop = LoadLoop("MOD/scripts/laser-loop.ogg")
	laserHitLoop = LoadLoop("MOD/scripts/laser-hit-loop.ogg")
	laserHitSound = LoadSound("MOD/scripts/spark0.ogg")
	laserDist = 0
	laserHitScale = 0

	laserSprite = LoadSprite("MOD/scripts/laser.png")
	setEnabled(false)
end

function setEnabled(e)
	enabled = e
	if enabled then
		SetShapeEmissiveScale(emitterShape, 1)
	else
		SetShapeEmissiveScale(emitterShape, 0)
	end
end

function emitSmoke(pos, amount)
	ParticleReset()
	ParticleType("smoke")
	ParticleColor(0.8, 0.8, 0.8)
	ParticleRadius(0.2, 0.4)
	ParticleAlpha(0.5, 0)
	ParticleDrag(0.5)
	ParticleGravity(rnd(0.0, 2.0))
	SpawnParticle(VecAdd(pos, rndVec(0.01)), rndVec(0.1), rnd(1.0, 3.0))

	ParticleReset()
	ParticleEmissive(5, 0, "easeout")
	ParticleGravity(-10)
	ParticleRadius(0.01, 0.0, "easein")
	ParticleColor(1, 0.4, 0.3)
	ParticleTile(4)
	local vel = VecAdd(Vec(0, 1, 0), rndVec(2.0))
	SpawnParticle(pos, vel, rnd(1.0, 2.0))
end

function getDistanceToLineSegment(point, p0, p1)
    local line_vec = VecSub(p1, p0)
    local pnt_vec = VecSub(point, p0)
    local line_len = VecLength(line_vec)
    local line_unitvec = VecNormalize(line_vec)
    local pnt_vec_scaled = VecScale(pnt_vec, 1.0/line_len)
    local t = VecDot(line_unitvec, pnt_vec_scaled)    
    if t < 0.0 then
        t = 0.0
    elseif t > 1.0 then
        t = 1.0
	end
    local nearest = VecScale(line_vec, t)
    local dist = VecLength(VecSub(nearest, pnt_vec))
    local nearest = VecAdd(nearest, p0)
    return dist, nearest
end

function shootLaser(body, shape, origin, dir, currentLength, deflectorsHit)
	--Raycast
	QueryRequire("physical")
	if shape ~= 0 then
		QueryRejectShape(shape)
	end
	if body ~= 0 then
		QueryRejectBody(body)
	end
	local hit, hitDist, hitNormal, hitShape = QueryRaycast(origin, dir, MAX_DIST, 0.0, true)
	local length = MAX_DIST
	if hit then
		length = hitDist
	end
	local hitPoint = VecAdd(origin, VecScale(dir, length))
	
	local t = Transform(VecLerp(origin, hitPoint, 0.5))
	local xAxis = VecNormalize(VecSub(hitPoint, origin))
	local zAxis = VecNormalize(VecSub(origin, GetCameraTransform().pos))
	t.rot = QuatAlignXZ(xAxis, zAxis)
	DrawSprite(laserSprite, t, length, 0.05+math.random()*0.01, 8, 4, 4, 1, true, true)
	DrawSprite(laserSprite, t, length, 0.5, 1.0, 0.3, 0.3, 1, true, true)

	--Check if player is hit by laser
	local ppos = GetPlayerCameraTransform().pos
	ppos[2] = ppos[2] - PLAYER_HIT_RADIUS*0.5
	local pdist, phit = getDistanceToLineSegment(ppos, origin, hitPoint)
	if pdist < PLAYER_HIT_RADIUS then
		--Decrease health but don't kill
		local health = GetPlayerHealth()
		health = math.max(0.1, health - 0.3)
		SetPlayerHealth(health)
		ReleasePlayerGrab()
		
		--Disance to player, without vertical component
		local pdir = VecSub(ppos, phit)
		pdir[2] = 0
		pdir = VecNormalize(pdir)

		--Move player and give a nudge
		local pt = GetPlayerTransform(true)
		pt.pos = VecAdd(pt.pos, VecScale(pdir, PLAYER_HIT_RADIUS-pdist+0.1))
		SetPlayerTransform(pt, true)
		SetPlayerVelocity(VecScale(pdir, 10))

		--Play hit sound
		PlaySound(laserHitSound, ppos)
	end
	
	--Check if directed
	local hitBody = GetShapeBody(hitShape)
	if HasTag(hitBody, "mirror2") and currentLength < MAX_DIST then
		--Only allow the beam to hit a deflector once
		local alreadyHit = false
		for i=1, #deflectorsHit do
			if deflectorsHit[i] == hitBody then
				alreadyHit = true
				break
			end
		end
		if not alreadyHit then
			deflectorsHit[#deflectorsHit+1] = hitBody
			local t = GetBodyTransform(hitBody)
			local refDir = TransformToParentVec(t, Vec(0, 0, 1))
			SetShapeEmissiveScale(hitShape, 1)
			return shootLaser(hitBody, hitShape, t.pos, refDir, currentLength + length, deflectorsHit)
		end
	end

	--Check if reflected
	local hitBody = GetShapeBody(hitShape)
	if HasTag(hitBody, "mirror") and currentLength < MAX_DIST then
		local refDir = VecSub(dir, VecScale(hitNormal, VecDot(hitNormal, dir)*2))
		return shootLaser(hitBody, hitShape, hitPoint, refDir, currentLength + length, deflectorsHit)
	end

	return currentLength + length, hitPoint, hitBody, hitShape
end

function tick(dt)
	if GetPlayerVehicle() == vehicle and InputPressed("lmb") then
		setEnabled(not enabled)
	end

	if enabled then
		local emitTransform = GetShapeWorldTransform(emitterShape)
		emitTransform.rot = QuatRotateQuat(emitTransform.rot, QuatEuler(90, 0, 0))
		local mi, ma = GetShapeBounds(emitterShape)
		local origin = VecLerp(mi, ma, 0.5)
		PlayLoop(laserLoop, origin, 0.5)
		local dir = TransformToParentVec(emitTransform, Vec(0, 0, -1))
		local length, hitPoint, hitBody, hitShape = shootLaser(0, emitterShape, origin, dir, 0, {})
		if length ~= laserDist then
			laserHitScale = 1
			laserDist = length
		end
		laserHitScale = math.max(0.0, laserHitScale - dt)
		if laserHitScale > 0 then
			PlayLoop(laserHitLoop, hitPoint, laserHitScale)
			PointLight(hitPoint, 1, 0.2, 0.2, rnd(2.0, 4.0)*laserHitScale)
		end
		PointLight(hitPoint, 1, 0.2, 0.2, rnd(0.5, 1.0))
	
		emitSmoke(hitPoint, 1.0)
		MakeHole(hitPoint, 0.5, 0.3, 0, true)
	end
end
