function VecDist(a, b)
	return VecLength(VecSub(a, b))
end

function clamp(value, mi, ma)
	if value < mi then value = mi end
	if value > ma then value = ma end
	return value
end

-- Angle between vectors, compensating for the inverse of the length
function VecPerpDot(a, b)
	b = VecScale(VecNormalize(b), -math.log(0.2 * clamp(VecLength(b), 0, 5) + 1, 2) + 1)
	--b = VecScale(VecNormalize(b), 2^-VecLength(b))
	--b = VecScale(VecNormalize(b), clamp(-VecLength(b) / 5 + 1, 0.1, 1))
	return a[1] * b[3] - a[3] * b[1]
end

function init()
	index = 1
	path = {}
	stop = true
	found = true
	next_index = 1
	path_length = 2.5
	viewDistance = 10
	obstacle_angle = 0
	vehicle = FindVehicle("van")
	checkpoints = FindLocations("checkpoint")
end

function SearchPath()
	stop = false
	found = false
	start_pos = GetVehicleTransform(vehicle).pos
	if next_index > 1 then
		start_pos = path[next_index-1]
	end
	if index > #checkpoints then
		index = 1
	end
	end_pos = GetLocationTransform(checkpoints[index]).pos
	QueryRejectVehicle(vehicle)
	QueryPath(start_pos, end_pos, 2 * VecDist(end_pos, start_pos), 2.0, "low")
end

function InitPath()
	if stop and found then
		SearchPath()
	end
	if not found and GetPathState() == "done" then
		for l = path_length, GetPathLength(), path_length do
			table.insert(path, GetPathPoint(l))
		end
		index = index + 1
		found = true
	elseif not found and GetPathState() == "fail" then
		SearchPath()
	end
end

function DriveVehicleTowardsPoint(vehicle, target)
	local vehicle_tr = GetVehicleTransform(vehicle)
	local vehicle_fwd = TransformToParentPoint(vehicle_tr, Vec(0, 0, -1))
	local fwd_vec = VecSub(vehicle_fwd, vehicle_tr.pos)
	local target_vec = VecSub(target, vehicle_tr.pos)
	local diff_angle = VecPerpDot(fwd_vec, target_vec)
	local steering = clamp(5 * diff_angle - obstacle_angle, -1, 1)
	DriveVehicle(vehicle, 1 - clamp(math.abs(steering), 0, 0.9), -steering, not path[next_index + 1])
end

function tick(dt)
	if GetPlayerVehicle() == vehicle then
		SetBool("level.ai_enabled", false)
		return
	end
	SetBool("level.ai_enabled", true)

	local vehicle_tr = GetVehicleTransform(vehicle)
	local vehicle_pos = vehicle_tr.pos
	local fwd = TransformToParentVec(vehicle_tr, Vec(0, 0, -1))
	QueryRequire("dynamic large")
	QueryRejectVehicle(vehicle)
	local hit, point, _, shape = QueryClosestPoint(vehicle_pos, 50)
	local to_obstacle = VecSub(point, vehicle_pos)
	local dist = VecLength(to_obstacle)
	obstacle_angle = 0
	if hit and dist < viewDistance then
		if VecDot(to_obstacle, fwd) > math.cos(45 * math.pi / 180) then
			obstacle_angle = VecPerpDot(fwd, to_obstacle)
			--DrawShapeHighlight(shape, 1)
		end
	end
	InitPath()
	if found and not stop then
		while VecLength(VecSub(vehicle_pos, path[next_index])) < 10 do
			next_index = next_index + 1
		end
		target = path[next_index]
		if target then
			DriveVehicleTowardsPoint(vehicle, target)
		else
			stop = true
			DriveVehicle(vehicle, 0.1, 0, true)
		end
	else
		DriveVehicle(vehicle, 0.1, 0, true)
	end
end
